/*            Copyright (C) 2000, 2001, 2002, 2003 Stijn van Dongen
 *
 * This file is part of MCL.  You can redistribute and/or modify MCL under the
 * terms of the GNU General Public License; either version 2 of the License or
 * (at your option) any later version.  You should have received a copy of the
 * GPL along with MCL, in the file COPYING.
*/

/* STATUS:
 * usable:  yes
 * ad hoc:  somewhat
 * quirks:  probably a few
 * support: limited
*/

#ifndef util_file_h
#define util_file_h

#include <stdio.h>

#include "ting.h"
#include "types.h"


#define MCX_READLINE_DEFAULT      0
#define MCX_READLINE_CHOMP        1
#define MCX_READLINE_SKIP_EMPTY   2
#define MCX_READLINE_PAR          4
#define MCX_READLINE_BSC          8
#define MCX_READLINE_DOT          16


/* as long as you did not use mcxIOopen,
 * feel free to do anything with the fn member,
 * especially right after mcxIOnew.
*/

typedef struct
{  mcxTing*       fn
;  const char*    mode
;  FILE*          fp
;  int            lc                /*    line count     */
;  int            lo                /*    line offset, corrupt after stepback */
;  int            bc                /*    byte count     */
;  int            ateof
;  int            stdio
;  void*          ufo               /*    user fondled object */
;
}  mcxIO    ;


/*
 *    mcxIOrenew does *not* support callback for resetting the ufo object
*/

mcxIO* mcxIOrenew
(  mcxIO*         xf
,  const char*    name
,  const char*    mode
)  ;


mcxIO* mcxIOnew
(  const char*    name
,  const char*    mode
)  ;

/*
 *    mcxIOfree does *not* support callback for freeing the ufo object
*/

void mcxIOfree
(  mcxIO**  xf
)  ;


void mcxIOfree_v
(  void*  xfpp
)  ;  

void mcxIOrelease
(  mcxIO*   xf
)  ;


mcxstatus mcxIOopen
(  mcxIO*   xf
,  mcxOnFail      ON_FAIL
)  ;


void mcxIOerr
(  mcxIO*   xf
,  const char     *complainer
,  const char     *complaint
)  ;


void mcxIOclose
(  mcxIO       *xf
)  ;


mcxTing*  mcxIOreadFile
(  mcxIO       *xf
,  mcxTing     *fileTxt
)  ;


/* NOTE.
 *    If this routine returns a NULL pointer it also frees the memory
 *    in  lineTxt. The callers argument then points to garbage.
 *    There is only one reasonable way of using this in a while loop:
 *
 *    # don't do this.
 *    # line is properly initialized and not NULL at this point.
 *    while (mcxIOreadLine(xf, line, flags))
 *    {  do stuff
 *    }
 *    # line now points to garbage
 *
 *    # do this instead
 *    # line is either NULL or is properly initialized.
 *    while ((line = mcxIOreadLine(xf, line, flags)))
 *    {  do stuff
 *    }
 *    # line now is NULL.
*/

mcxTing*  mcxIOreadLine
(  mcxIO       *xf
,  mcxTing     *lineTxt
,  mcxbits     flags
)  ;


/*    returns count of discarded characters.
*/

int mcxIOdiscardLine
(  mcxIO       *xf
)  ;


/* OK to call this after mcxIOnew, before mcxIOopen */

mcxstatus mcxIOnewName
(  mcxIO*    xf
,  const char* newname
)  ;


/* OK to call this after mcxIOnew, before mcxIOopen */

mcxstatus mcxIOappendName
(  mcxIO*    xf
,  const char* suffix
)  ;


int mcxIOstep
(  mcxIO*    xf
)  ;


int mcxIOstepback
(  int c
,  mcxIO*    xf
)  ;


void mcxIOpos
(  mcxIO*   xf
,  FILE*          channel
)  ;


void mcxIOlistParmodes
(  void
)  ;

void mcxIOrewind
(  mcxIO*   xf
)  ;


/*    returns STATUS_OK if str is encountered in stream at this position,
 *    STATUS_FAIL otherwise.  When failing and if stream is *not* stdin, the
 *    file pointer is reset to where it was. When failing and stream *is*
 *    stdin, file pointer is not reset. So you'll lose when searching for
 *    strstrx in strstrstrx - Because the leading strstr is slurped; it is
 *    noted that the next 's' is not an 'x', and the search is restarted with
 *    the remaining 'strx' part.  This sucks almost beyond comprehension, but
 *    there you are.
 *   
 *    As for the check whether str is stdin, this may need fixing too - not yet
 *    sure what happens when file descriptors are dupped.  Should I check for
 *    fileno(fp) ?
 *       
 *    As the quality of these comments indicate, this code needs a fix badly;
 *    don't use it unless you are the braindamaged maintainer.
*/

mcxstatus mcxIOexpect
(  mcxIO*               xf
,  const char*          str
,  mcxOnFail            ON_FAIL
)  ;

mcxstatus mcxIOexpectReal
(  mcxIO*        xf
,  double*       dblp
,  mcxOnFail     ON_FAIL
)  ;

mcxstatus mcxIOexpectNum
(  mcxIO*        xf
,  long*         lngp
,  mcxOnFail     ON_FAIL
)  ;


/*
 *    Returns next non-white space char,
 *    which is pushed back onto stream after reading.
*/

int mcxIOskipSpace
(  mcxIO*               xf
)  ;


/*
 *    Purpose: find str in file, period.
 *
 *    This one below is BROKEN for certain string/text combinations.
 *    E.g. searching for strstrx in strstrstrx will fail.
 *    Has to do with not seeking in file; which is because
 *    STDIN is "supported".
 *
 *    Hum ho, the code now checks whether the stream is a standard IO stream
 *    (c.q. stdin). However, I dunnow how this behaves when you are
 *    replumbing your pipes with dup2 and friends.
 *    Should I check for fileno(fp) ?
 *    
 *    As the quality of these comments indicate, this code needs a fix badly;
 *    don't use it unless you are the braindamaged maintainer.
*/

mcxstatus mcxIOfind
(  mcxIO*               xf
,  const char*          str
,  mcxOnFail            ON_FAIL
)  ;




/* 
 * The stuff below still needs to be cast into the mcxIO framework as can be
 * seen from FILE* ptrs.  It is very skinny and obsolete; it embodies support
 * for binary IO (fwrite, fread), but it should be reworked one day once
 * somebody actually needs it.
*/

int mcxIOexpectCanary
(  FILE*                f_in
,  int                  number
)  ;

void mcxIOwriteCanary
(  FILE*                f_out
,  int                  number
)  ;

int mcxIOreadInteger
(  FILE*                f_in
)  ;

int mcxIOwriteInteger
(  FILE*                f_out
,  int                  val
)  ;


#endif

